@@ -44,21 +44,35 @@
 
 static GtkBuilder *builder = NULL;
 static GtkListStore *list_store = NULL;
-static PkClient *client = NULL;
+static PkClient *client_query = NULL;
+#if PK_CHECK_VERSION(0,5,1)
+static PkClientPool *pool = NULL;
+#else
+static GPtrArray *client_array = NULL;
+#endif
 static PkBitfield roles;
 static GConfClient *gconf_client;
 static gboolean show_details;
 static GtkTreePath *path_global = NULL;
 static GtkWidget *image_animation = NULL;
+#if !PK_CHECK_VERSION(0,5,1)
+static PkStatusEnum status_last = PK_STATUS_ENUM_UNKNOWN;
+#endif
+static guint status_id = 0;
 
 enum {
 	REPO_COLUMN_ENABLED,
 	REPO_COLUMN_TEXT,
 	REPO_COLUMN_ID,
 	REPO_COLUMN_ACTIVE,
+	REPO_COLUMN_SENSITIVE,
 	REPO_COLUMN_LAST
 };
 
+#if !PK_CHECK_VERSION(0,5,1)
+static PkClient *gpk_repo_create_client (void);
+#endif
+
 /**
  * gpk_repo_find_iter_model_cb:
  **/
@@ -151,16 +165,16 @@ gpk_button_help_cb (GtkWidget *widget, gboolean  data)
 }
 
 static void
-gpk_misc_installed_toggled (GtkCellRendererToggle *cell, gchar *path_str, gpointer data)
+gpk_misc_enabled_toggled (GtkCellRendererToggle *cell, gchar *path_str, gpointer data)
 {
-	GtkWidget *widget;
 	GtkTreeModel *model = (GtkTreeModel *)data;
 	GtkTreeIter iter;
 	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
-	gboolean installed;
+	gboolean enabled;
 	gchar *repo_id;
 	gboolean ret;
 	GError *error = NULL;
+	PkClient *client;
 
 	/* do we have the capability? */
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_REPO_ENABLE) == FALSE) {
@@ -168,36 +182,34 @@ gpk_misc_installed_toggled (GtkCellRendererToggle *cell, gchar *path_str, gpoint
 		return;
 	}
 
-	/* set insensitive until we've done this */
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_repo"));
-	gtk_widget_set_sensitive (widget, FALSE);
-
 	/* get toggled iter */
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get (model, &iter,
-			    REPO_COLUMN_ENABLED, &installed,
+			    REPO_COLUMN_ENABLED, &enabled,
 			    REPO_COLUMN_ID, &repo_id, -1);
 
 	/* do something with the value */
-	installed ^= 1;
+	enabled ^= 1;
 
 	/* do this to the repo */
-	egg_debug ("setting %s to %i", repo_id, installed);
-	ret = pk_client_reset (client, &error);
-	if (!ret) {
-		egg_warning ("failed to reset client: %s", error->message);
-		g_error_free (error);
-		goto out;
-	}
-	ret = pk_client_repo_enable (client, repo_id, installed, &error);
+	egg_debug ("setting %s to %i", repo_id, enabled);
+#if PK_CHECK_VERSION(0,5,1)
+	client = pk_client_pool_create (pool);
+#else
+	client = gpk_repo_create_client ();
+#endif
+	ret = pk_client_repo_enable (client, repo_id, enabled, &error);
 	if (!ret) {
 		egg_warning ("could not set repo enabled state: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
+	g_object_unref (client);
 
 	/* set new value */
-	gtk_list_store_set (GTK_LIST_STORE(model), &iter, REPO_COLUMN_ENABLED, installed, -1);
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			    REPO_COLUMN_SENSITIVE, FALSE,
+			    -1);
 
 out:
 	/* clean up */
@@ -209,7 +221,7 @@ out:
  * gpk_repo_detail_cb:
  **/
 static void
-gpk_repo_detail_cb (PkClient *client_, const gchar *repo_id,
+gpk_repo_detail_cb (PkClient *client, const gchar *repo_id,
 		    const gchar *description, gboolean enabled, gpointer data)
 {
 	GtkTreeIter iter;
@@ -224,6 +236,7 @@ gpk_repo_detail_cb (PkClient *client_, const gchar *repo_id,
 			    REPO_COLUMN_TEXT, description,
 			    REPO_COLUMN_ID, repo_id,
 			    REPO_COLUMN_ACTIVE, TRUE,
+			    REPO_COLUMN_SENSITIVE, TRUE,
 			    -1);
 
 	/* sort after each entry, which is okay as there shouldn't be many */
@@ -240,13 +253,15 @@ gpk_treeview_add_columns (GtkTreeView *treeview)
 	GtkTreeViewColumn *column;
 	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
 
-	/* column for installed toggles */
+	/* column for enabled toggles */
 	renderer = gtk_cell_renderer_toggle_new ();
-	g_signal_connect (renderer, "toggled", G_CALLBACK (gpk_misc_installed_toggled), model);
+	g_signal_connect (renderer, "toggled", G_CALLBACK (gpk_misc_enabled_toggled), model);
 
 	/* TRANSLATORS: column if the source is enabled */
 	column = gtk_tree_view_column_new_with_attributes (_("Enabled"), renderer,
-							   "active", REPO_COLUMN_ENABLED, NULL);
+							   "active", REPO_COLUMN_ENABLED,
+							   "sensitive", REPO_COLUMN_SENSITIVE,
+							   NULL);
 	gtk_tree_view_append_column (treeview, column);
 
 	/* column for text */
@@ -282,15 +297,10 @@ gpk_repos_treeview_clicked_cb (GtkTreeSelection *selection, gpointer data)
  * gpk_repo_finished_cb:
  **/
 static void
-gpk_repo_finished_cb (PkClient *client_, PkExitEnum exit, guint runtime, gpointer data)
+gpk_repo_finished_cb (PkClient *client, PkExitEnum exit, guint runtime, gpointer data)
 {
 	GtkTreeView *treeview;
 	GtkTreeModel *model;
-	GtkWidget *widget;
-
-	/* set sensitive now we've done this */
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_repo"));
-	gtk_widget_set_sensitive (widget, TRUE);
 
 	/* remove the items that are not used */
 	treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview_repo"));
@@ -299,22 +309,26 @@ gpk_repo_finished_cb (PkClient *client_, PkExitEnum exit, guint runtime, gpointe
 }
 
 /**
- * gpk_repo_status_changed_cb:
+ * gpk_repo_status_changed_timeout_cb:
  **/
-static void
-gpk_repo_status_changed_cb (PkClient *client_, PkStatusEnum status, gpointer data)
+static gboolean
+gpk_repo_status_changed_timeout_cb (PkClient *client)
 {
 	const gchar *text;
 	GtkWidget *widget;
+	PkStatusEnum status;
 
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "viewport_animation_preview"));
-	if (status == PK_STATUS_ENUM_FINISHED) {
-		gtk_widget_hide (widget);
-		gpk_animated_icon_enable_animation (GPK_ANIMATED_ICON (image_animation), FALSE);
-		return;
-	}
+#if PK_CHECK_VERSION(0,5,1)
+	/* get the last status */
+	g_object_get (client,
+		      "status", &status,
+		      NULL);
+#else
+	status = status_last;
+#endif
 
 	/* set the text and show */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "viewport_animation_preview"));
 	gtk_widget_show (widget);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_animation"));
 	text = gpk_status_enum_to_localised_text (status);
@@ -322,14 +336,51 @@ gpk_repo_status_changed_cb (PkClient *client_, PkStatusEnum status, gpointer dat
 
 	/* set icon */
 	gpk_set_animated_icon_from_status (GPK_ANIMATED_ICON (image_animation), status, GTK_ICON_SIZE_LARGE_TOOLBAR);
-	gtk_widget_show (widget);
+
+	/* never repeat */
+	status_id = 0;
+	return FALSE;
+}
+
+/**
+ * gpk_repo_status_changed_cb:
+ **/
+static void
+gpk_repo_status_changed_cb (PkClient *client, PkStatusEnum status, gpointer data)
+{
+	GtkWidget *widget;
+
+	if (status == PK_STATUS_ENUM_FINISHED) {
+		/* we've not yet shown, so don't bother */
+		if (status_id > 0) {
+			g_source_remove (status_id);
+			status_id = 0;
+		}
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "viewport_animation_preview"));
+		gtk_widget_hide (widget);
+		gpk_animated_icon_enable_animation (GPK_ANIMATED_ICON (image_animation), FALSE);
+		goto out;
+	}
+
+	/* already pending show */
+	if (status_id > 0)
+		goto out;
+
+	/* only show after some time in the transaction */
+	status_id = g_timeout_add (GPK_UI_STATUS_SHOW_DELAY, (GSourceFunc) gpk_repo_status_changed_timeout_cb, client);
+out:
+#if !PK_CHECK_VERSION(0,5,1)
+	/* save for the callback */
+	status_last = status;
+#endif
+	return;
 }
 
 /**
  * gpk_repo_error_code_cb:
  **/
 static void
-gpk_repo_error_code_cb (PkClient *client_, PkErrorCodeEnum code, const gchar *details, gpointer data)
+gpk_repo_error_code_cb (PkClient *client, PkErrorCodeEnum code, const gchar *details, gpointer data)
 {
 	GtkWindow *window;
 	window = GTK_WINDOW (gtk_builder_get_object (builder, "dialog_repo"));
@@ -356,7 +407,7 @@ gpk_repo_repo_list_refresh (void)
 	gpk_repo_mark_nonactive (model);
 
 	egg_debug ("refreshing list");
-	ret = pk_client_reset (client, &error);
+	ret = pk_client_reset (client_query, &error);
 	if (!ret) {
 		egg_warning ("failed to reset client: %s", error->message);
 		g_error_free (error);
@@ -366,7 +417,7 @@ gpk_repo_repo_list_refresh (void)
 		filters = pk_bitfield_value (PK_FILTER_ENUM_NOT_DEVELOPMENT);
 	else
 		filters = pk_bitfield_value (PK_FILTER_ENUM_NONE);
-	ret = pk_client_get_repo_list (client, filters, &error);
+	ret = pk_client_get_repo_list (client_query, filters, &error);
 	if (!ret) {
 		egg_warning ("failed to get repo list: %s", error->message);
 		g_error_free (error);
@@ -407,6 +458,44 @@ gpk_repo_message_received_cb (UniqueApp *app, UniqueCommand command, UniqueMessa
 	}
 }
 
+#if !PK_CHECK_VERSION(0,5,1)
+/**
+ * gpk_repo_destroy_cb:
+ **/
+static void
+gpk_repo_destroy_cb (PkClient *client, gpointer data)
+{
+	gboolean ret;
+	egg_debug ("client destroyed");
+	ret = g_ptr_array_remove (client_array, client);
+	if (!ret)
+		egg_warning ("failed to remove %p", client);
+	/* TODO: disconnect signals? */
+	g_object_unref (client);
+}
+
+/**
+ * gpk_repo_create_client
+ **/
+static PkClient *
+gpk_repo_create_client (void)
+{
+	PkClient *client;
+	client = pk_client_new ();
+	g_signal_connect (client, "repo-detail",
+			  G_CALLBACK (gpk_repo_detail_cb), NULL);
+	g_signal_connect (client, "status-changed",
+			  G_CALLBACK (gpk_repo_status_changed_cb), NULL);
+	g_signal_connect (client, "error-code",
+			  G_CALLBACK (gpk_repo_error_code_cb), NULL);
+	g_signal_connect (client, "destroy",
+			  G_CALLBACK (gpk_repo_destroy_cb), NULL);
+	g_ptr_array_add (client_array, client);
+	egg_debug ("added %p", client);
+	return g_object_ref (client);
+}
+#endif
+
 /**
  * main:
  **/
@@ -476,16 +565,28 @@ main (int argc, char *argv[])
 
 	gconf_client = gconf_client_get_default ();
 
-	client = pk_client_new ();
-	g_signal_connect (client, "repo-detail",
+	client_query = pk_client_new ();
+	g_signal_connect (client_query, "repo-detail",
 			  G_CALLBACK (gpk_repo_detail_cb), NULL);
-	g_signal_connect (client, "status-changed",
+	g_signal_connect (client_query, "status-changed",
 			  G_CALLBACK (gpk_repo_status_changed_cb), NULL);
-	g_signal_connect (client, "finished",
+	g_signal_connect (client_query, "finished",
 			  G_CALLBACK (gpk_repo_finished_cb), NULL);
-	g_signal_connect (client, "error-code",
+	g_signal_connect (client_query, "error-code",
 			  G_CALLBACK (gpk_repo_error_code_cb), NULL);
 
+#if PK_CHECK_VERSION(0,5,1)
+	pool = pk_client_pool_new ();
+	pk_client_pool_connect (pool, "repo-detail",
+				G_CALLBACK (gpk_repo_detail_cb), NULL);
+	pk_client_pool_connect (pool, "status-changed",
+				G_CALLBACK (gpk_repo_status_changed_cb), NULL);
+	pk_client_pool_connect (pool, "error-code",
+				G_CALLBACK (gpk_repo_error_code_cb), NULL);
+#else
+	client_array = g_ptr_array_new ();
+#endif
+
 	control = pk_control_new ();
 	g_signal_connect (control, "repo-list-changed",
 			  G_CALLBACK (gpk_repo_repo_list_changed_cb), NULL);
@@ -528,7 +629,7 @@ main (int argc, char *argv[])
 
 	/* create list stores */
 	list_store = gtk_list_store_new (REPO_COLUMN_LAST, G_TYPE_BOOLEAN,
-					 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+					 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
 
 	/* create repo tree view */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_repo"));
@@ -560,7 +661,7 @@ main (int argc, char *argv[])
 		/* get the update list */
 		gpk_repo_repo_list_refresh ();
 	} else {
-		gpk_repo_detail_cb (client, "default",
+		gpk_repo_detail_cb (client_query, "default",
 				   _("Getting software source list not supported by backend"), FALSE, NULL);
 		widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_repo"));
 		gtk_widget_set_sensitive (widget, FALSE);
@@ -575,8 +676,14 @@ main (int argc, char *argv[])
 out_build:
 	g_object_unref (builder);
 	g_object_unref (gconf_client);
-	g_object_unref (client);
+	g_object_unref (client_query);
 	g_object_unref (control);
+#if PK_CHECK_VERSION(0,5,1)
+	g_object_unref (pool);
+#else
+	g_ptr_array_foreach (client_array, (GFunc) g_object_unref, NULL);
+	g_ptr_array_free (client_array, TRUE);
+#endif
 unique_out:
 	g_object_unref (unique_app);
 
